from dataclasses import dataclass
from edinet.financial.statements import Statements as Statements
from edinet.models.financial import LineItem as LineItem
from edinet.xbrl._namespaces import NamespaceInfo as NamespaceInfo, classify_namespace as classify_namespace, is_standard_taxonomy as is_standard_taxonomy
from edinet.xbrl.linkbase.calculation import CalculationLinkbase as CalculationLinkbase
from edinet.xbrl.linkbase.definition import DefinitionTree as DefinitionTree

@dataclass(frozen=True, slots=True)
class _CalcMappingResult:
    """CalculationLinkbase ベースのマッピング結果（内部用）。

    Attributes:
        ancestor: summation-item 由来の最近接標準科目のローカル名。
            標準科目が見つからない場合は None。
        path: 非標準科目からの経路（ローカル名のタプル）。
        role_uri: マッピングが特定された role URI。
    """
    ancestor: str | None
    path: tuple[str, ...]
    role_uri: str

@dataclass(frozen=True, slots=True)
class CustomItemInfo:
    """非標準（拡張）科目の分析結果。

    Attributes:
        item: 元の LineItem への参照。
        namespace_info: 名前空間の分類結果（カテゴリ、EDINET コード等）。
        parent_standard_concept: general-special arcrole で推定した
            親標準科目のローカル名。DefinitionLinkbase が未指定、
            または該当する関係がない場合は None。
        calculation_ancestor: Calculation Linkbase (summation-item) 由来の
            最近接標準科目のローカル名。CalculationLinkbase が未指定、
            または該当する計算関係がない場合は None。
        calculation_path: 非標準科目から calculation_ancestor までの経路
            （ローカル名のタプル）。先頭が非標準科目、末尾が
            calculation_ancestor。calculation_ancestor が None の場合は
            ルートまでの経路。CalculationLinkbase が未指定の場合は None。
        calculation_role_uri: マッピングが特定された Calculation Linkbase の
            role URI。CalculationLinkbase が未指定の場合は None。
    """
    item: LineItem
    namespace_info: NamespaceInfo
    parent_standard_concept: str | None
    calculation_ancestor: str | None = ...
    calculation_path: tuple[str, ...] | None = ...
    calculation_role_uri: str | None = ...

@dataclass(frozen=True, slots=True)
class CustomDetectionResult:
    """標準/非標準科目の分類結果。

    Attributes:
        custom_items: 非標準科目の分析結果タプル。
        standard_items: 標準科目の LineItem タプル。
        custom_ratio: 非標準科目の割合（0.0〜1.0）。
            全科目数が 0 の場合は 0.0。Fact 単位の割合であり、
            ユニーク concept 数ベースの割合ではない。
        total_count: 全科目数（custom + standard）。
    """
    custom_items: tuple[CustomItemInfo, ...]
    standard_items: tuple[LineItem, ...]
    custom_ratio: float
    total_count: int

def detect_custom_items(statements: Statements, *, definition_linkbase: dict[str, DefinitionTree] | None = None, calculation_linkbase: CalculationLinkbase | None = None) -> CustomDetectionResult:
    """各 LineItem を標準/非標準に分類する。

    Statements 内の全 LineItem について、名前空間 URI を基に
    標準タクソノミか提出者別タクソノミかを判定する。
    非標準科目については NamespaceInfo と、Definition Linkbase が
    利用可能な場合は親標準科目の推定結果を付与する。

    Calculation Linkbase が指定された場合は、summation-item arcrole を
    辿って「どの標準科目の計算内訳か」も推定する。

    Note:
        本関数は XBRL インスタンス内の Fact（LineItem）のみを対象とする。
        タクソノミ定義上の abstract 要素（domainItemType のセグメント Member 等）は
        XBRL インスタンスに Fact として出現しないため検出対象外である。

        同一 concept が複数の期間・次元に出現する場合、各 Fact は独立にカウント
        される。custom_ratio は Fact 単位の割合であり、ユニーク concept 数ベース
        の割合ではない。concept レベルの割合が必要な場合は
        ``{ci.item.local_name for ci in result.custom_items}`` で重複除去すること。

    Args:
        statements: build_statements() で構築した Statements。
        definition_linkbase: parse_definition_linkbase() の戻り値。
            指定した場合、general-special arcrole を用いて
            拡張科目の親標準科目を推定する。None の場合は推定しない。
        calculation_linkbase: parse_calculation_linkbase() の戻り値。
            指定した場合、summation-item arcrole を用いて
            拡張科目の計算上の最近接標準科目を推定する。
            None の場合は推定しない。

    Returns:
        CustomDetectionResult。
    """
def find_custom_concepts(calc_linkbase: CalculationLinkbase) -> tuple[str, ...]:
    """Calculation Linkbase 内の全非標準科目を自動検出する。

    全 CalculationArc の parent/child の href を走査し、
    提出者タクソノミに属する concept を抽出する。

    Args:
        calc_linkbase: パース済みの CalculationLinkbase。

    Returns:
        非標準科目ローカル名のタプル（重複除去・ソート済み）。
    """



def normalize_edinet_code(value: str | None, *, allow_none: bool = True) -> str | None:
    """edinet_code を正規化して検証する。

    Args:
        value: 入力値。
        allow_none: `None` を許可する場合は `True`。

    Returns:
        正規化済みの EDINET コード。`allow_none=True` かつ `value=None` の場合は `None`。

    Raises:
        ValueError: 型不正、空文字、または `E` + 5桁形式に一致しない場合。
    """



from _typeshed import Incomplete
from dataclasses import dataclass
from edinet.exceptions import EdinetParseError as EdinetParseError, EdinetWarning as EdinetWarning
from edinet.xbrl._namespaces import NS_LINK as NS_LINK, NS_XBRLDT as NS_XBRLDT, NS_XLINK as NS_XLINK

logger: Incomplete

@dataclass(frozen=True, slots=True)
class DefinitionArc:
    '''Definition Linkbase の 1 本の arc。

    Attributes:
        from_concept: 親 concept のローカル名（例: ``"ConsolidatedBalanceSheetHeading"``）。
        to_concept: 子 concept のローカル名（例: ``"BalanceSheetTable"``）。
        from_href: 親 concept の xlink:href（XSD 相対パス + フラグメント）。
        to_href: 子 concept の xlink:href。
        arcrole: arcrole URI。
        order: 表示順。
        closed: xbrldt:closed 属性値。``all`` arcrole のアークでのみ設定される。
            それ以外のアークでは**常に** ``None``（XML に属性が存在しても無視される）。
        context_element: xbrldt:contextElement 属性値。``all`` arcrole のアークでのみ設定される。
            それ以外のアークでは**常に** ``None``。
        usable: usable 属性値。全 arc 型の XML 属性から読み取るが、
            意味的に有効なのは ``domain-member`` arcrole のみ。
    '''
    from_concept: str
    to_concept: str
    from_href: str
    to_href: str
    arcrole: str
    order: float
    closed: bool | None = ...
    context_element: str | None = ...
    usable: bool = ...

@dataclass(frozen=True, slots=True)
class MemberNode:
    '''ドメイン配下のメンバーノード（ツリー構造）。

    ドメインをルートとし、domain-member arc による階層を再帰的に保持する。
    SS の列ヘッダー構築等で階層情報と usable フラグが必要。

    Attributes:
        concept: メンバーのローカル名（例: ``"ShareholdersEquityAbstract"``）。
        href: xlink:href（XSD 相対パス + フラグメント）。
        usable: Fact を持ちうるか。``False`` の場合は表示専用（Abstract 等）。
        children: 子メンバーノードのタプル（order 順ソート済み）。
    '''
    concept: str
    href: str
    usable: bool = ...
    children: tuple[MemberNode, ...] = ...

@dataclass(frozen=True, slots=True)
class AxisInfo:
    '''1 つの Axis（次元軸）の構造化情報。

    Attributes:
        axis_concept: Axis のローカル名（例: ``"ComponentsOfEquityAxis"``）。
        order: hypercube-dimension arc の order 値（複数 Axis の表示順序）。
        domain: ドメインのルートノード。子にメンバー階層を再帰的に持つ。
            ドメインが未定義の場合は None。
        default_member: デフォルトメンバーのローカル名。
            デフォルトがない場合は None。
    '''
    axis_concept: str
    order: float = ...
    domain: MemberNode | None = ...
    default_member: str | None = ...

@dataclass(frozen=True, slots=True)
class HypercubeInfo:
    '''ハイパーキューブの構造化情報。

    Attributes:
        table_concept: Table のローカル名。
        heading_concept: Heading（ルート要素）のローカル名。
        axes: ハイパーキューブに属する Axis の情報
            （``hypercube-dimension`` arc の order 昇順でソート済み）。
        closed: ハイパーキューブが closed かどうか。
        context_element: ``"segment"`` or ``"scenario"``。
        line_items_concept: LineItems のローカル名。
            LineItems がない場合は None。
    '''
    table_concept: str
    heading_concept: str
    axes: tuple[AxisInfo, ...]
    closed: bool
    context_element: str
    line_items_concept: str | None = ...

@dataclass(frozen=True, slots=True)
class DefinitionTree:
    """1 つの role URI に対応する定義ツリー。

    Attributes:
        role_uri: ロール URI。
        arcs: 全 arc のタプル。認識されない arcrole の arc も含む。
        hypercubes: 構造化されたハイパーキューブ情報のタプル。
            ``arcs`` の部分集合（``all``, ``hypercube-dimension``,
            ``dimension-domain``, ``dimension-default``, ``domain-member``
            arcrole）を構造化したもの。``general-special`` のみの role では
            空タプル（``has_hypercube`` == False）。
    """
    role_uri: str
    arcs: tuple[DefinitionArc, ...]
    hypercubes: tuple[HypercubeInfo, ...]
    @property
    def has_hypercube(self) -> bool:
        """ハイパーキューブ構造を含むかどうか。"""

def parse_definition_linkbase(xml_bytes: bytes, *, source_path: str | None = None) -> dict[str, DefinitionTree]:
    """Definition Linkbase の XML bytes をパースする。

    Args:
        xml_bytes: ``_def.xml`` の bytes。
        source_path: エラーメッセージに含めるソースパス（任意）。

    Returns:
        role_uri をキー、DefinitionTree を値とする辞書。

    Raises:
        EdinetParseError: XML パースに失敗した場合。
    """



from edinet.xbrl.contexts import ContextCollection as ContextCollection, structure_contexts as structure_contexts
from edinet.xbrl.dei import AccountingStandard as AccountingStandard, DEI as DEI, PeriodType as PeriodType, extract_dei as extract_dei
from edinet.xbrl.facts import build_line_items as build_line_items
from edinet.xbrl.ixbrl_parser import merge_ixbrl_results as merge_ixbrl_results, parse_ixbrl_facts as parse_ixbrl_facts
from edinet.xbrl.parser import parse_xbrl_facts as parse_xbrl_facts
from edinet.xbrl.taxonomy import TaxonomyResolver as TaxonomyResolver
from edinet.xbrl.taxonomy.custom import CustomDetectionResult as CustomDetectionResult, detect_custom_items as detect_custom_items, find_custom_concepts as find_custom_concepts
from edinet.xbrl.text import SectionMap as SectionMap, TextBlock as TextBlock, build_section_map as build_section_map, clean_html as clean_html, extract_text_blocks as extract_text_blocks
from edinet.xbrl.units import DivideMeasure as DivideMeasure, Measure as Measure, SimpleMeasure as SimpleMeasure, StructuredUnit as StructuredUnit, structure_units as structure_units
from edinet.xbrl.validation.calc_check import CalcValidationResult as CalcValidationResult, validate_calculations as validate_calculations

__all__ = ['parse_xbrl_facts', 'parse_ixbrl_facts', 'merge_ixbrl_results', 'structure_contexts', 'ContextCollection', 'TaxonomyResolver', 'build_line_items', 'build_statements', 'Statements', 'DEI', 'AccountingStandard', 'PeriodType', 'extract_dei', 'DetectedStandard', 'detect_accounting_standard', 'SimpleMeasure', 'DivideMeasure', 'Measure', 'StructuredUnit', 'structure_units', 'CustomDetectionResult', 'detect_custom_items', 'find_custom_concepts', 'CalcValidationResult', 'validate_calculations', 'FiscalYearInfo', 'detect_fiscal_year', 'TextBlock', 'extract_text_blocks', 'SectionMap', 'build_section_map', 'clean_html']

# Names in __all__ with no definition:
#   DetectedStandard
#   FiscalYearInfo
#   Statements
#   build_statements
#   detect_accounting_standard
#   detect_fiscal_year



import pandas as pd
from collections.abc import Sequence
from edinet.models.financial import LineItem

__all__ = ['line_items_to_dataframe']

def line_items_to_dataframe(items: Sequence[LineItem], *, metadata: dict[str, object] | None = None) -> pd.DataFrame:
    """全 LineItem を全カラム DataFrame に変換する。

    Args:
        items: LineItem のシーケンス。
        metadata: DataFrame の attrs に付与するメタデータ。

    Returns:
        全カラムの pandas DataFrame。

    Raises:
        ImportError: pandas がインストールされていない場合。
    """



from _typeshed import Incomplete
from edinet.exceptions import EdinetWarning as EdinetWarning
from pydantic import BaseModel

class FormCodeEntry(BaseModel):
    """様式コードリストの1行。"""
    model_config: Incomplete
    ordinance_code: str
    form_code: str
    form_number: str
    form_name: str
    doc_type_name: str
    disclosure_type: str
    note: str | None

FORM_CODE_RECORD_COUNT: Incomplete

def get_form_code(ordinance_code: str, form_code: str) -> FormCodeEntry | None:
    """府令コードと様式コードのペアから FormCodeEntry を返す。"""
def all_form_codes() -> list[FormCodeEntry]:
    """全様式コードの一覧を返す。"""



from _typeshed import Incomplete
from dataclasses import dataclass
from edinet.exceptions import EdinetParseError as EdinetParseError, EdinetWarning as EdinetWarning
from edinet.xbrl._linkbase_utils import ROLE_LABEL as ROLE_LABEL, ROLE_NEGATED_LABEL as ROLE_NEGATED_LABEL, ROLE_PERIOD_END_LABEL as ROLE_PERIOD_END_LABEL, ROLE_PERIOD_START_LABEL as ROLE_PERIOD_START_LABEL, ROLE_TERSE_LABEL as ROLE_TERSE_LABEL, ROLE_TOTAL_LABEL as ROLE_TOTAL_LABEL, ROLE_VERBOSE_LABEL as ROLE_VERBOSE_LABEL
from edinet.xbrl._namespaces import NS_LINK as NS_LINK, NS_XLINK as NS_XLINK

logger: Incomplete

@dataclass(frozen=True, slots=True)
class _LocInfo:
    """loc 要素から抽出した情報。

    Attributes:
        href: xlink:href 属性値（元の値）。
        concept: 正規化済み concept 名。
    """
    href: str
    concept: str

@dataclass(frozen=True, slots=True)
class _RawArc:
    """presentationArc 要素から抽出した情報。

    Attributes:
        from_label: xlink:from 属性値。
        to_label: xlink:to 属性値。
        order: order 属性値。
        preferred_label: preferredLabel 属性値。
    """
    from_label: str
    to_label: str
    order: float
    preferred_label: str | None

@dataclass(frozen=True, slots=True)
class PresentationNode:
    '''Presentation Linkbase のツリーノード。

    Attributes:
        concept: 正規化済み concept 名（例: "CashAndDeposits"）。
        href: 元の xlink:href 値。
        order: 表示順序。
        preferred_label: preferredLabel ロール URI。None の場合は標準ラベル。
        depth: ツリーにおける深さ（ルート = 0）。
        children: 子ノードのタプル（order 順）。
        is_abstract: Abstract ノードかどうか。concept 名が "Abstract" で
            終わるかどうかで判定する。
        is_dimension_node: ディメンション関連ノードかどうか。
    '''
    concept: str
    href: str
    order: float
    preferred_label: str | None = ...
    depth: int = ...
    children: tuple[PresentationNode, ...] = ...
    is_abstract: bool = ...
    is_dimension_node: bool = ...
    @property
    def is_total(self) -> bool:
        """合計行かどうかを返す。

        Returns:
            preferred_label が totalLabel ロールの場合 True。
        """

@dataclass(frozen=True, slots=True)
class PresentationTree:
    """1 つの role URI に対応する Presentation ツリー。

    Attributes:
        role_uri: ツリーが属する role URI。
        roots: ルートノード群（order 順）。
        node_count: ツリー内の全ノード数。
    """
    role_uri: str
    roots: tuple[PresentationNode, ...] = ...
    node_count: int = ...
    def line_items_roots(self) -> tuple[PresentationNode, ...]:
        """LineItems ノード以下のルートを返す。

        Presentation ツリーでは Heading → Table → LineItems の構造が
        一般的であり、実際の科目は LineItems の子として並ぶ。
        本メソッドは LineItems ノードを探索し、その子をルートとして返す。

        LineItems が見つからない場合は roots をそのまま返す。

        Returns:
            LineItems ノードの子、または roots。
        """
    def flatten(self, *, skip_abstract: bool = False, skip_dimension: bool = False) -> tuple[PresentationNode, ...]:
        """ツリーを深さ優先で平坦化する。

        Args:
            skip_abstract: True の場合、Abstract ノードをスキップする。
            skip_dimension: True の場合、ディメンション関連ノードをスキップする。

        Returns:
            深さ優先順のノードタプル。

        Note:
            同一 concept が複数の位置に出現する場合（例: dimension ノードと
            科目ツリーの両方に同じ concept が参照される場合）、返り値には
            同一 concept 名の異なるノードが含まれる。concept のユニーク
            集合が必要な場合は ``{n.concept for n in tree.flatten()}`` を使用する。
        """

def parse_presentation_linkbase(xml_bytes: bytes, *, source_path: str | None = None) -> dict[str, PresentationTree]:
    """Presentation Linkbase XML をパースし、role URI ごとのツリーを返す。

    Args:
        xml_bytes: linkbase XML のバイト列。
        source_path: エラーメッセージ用のソースファイルパス。

    Returns:
        role URI をキー、PresentationTree を値とする辞書。

    Raises:
        EdinetParseError: XML が不正な場合。
    """
def merge_presentation_trees(*tree_dicts: dict[str, PresentationTree]) -> dict[str, PresentationTree]:
    """複数の PresentationTree 辞書をマージする。

    同一 role URI のツリーは再帰的に子をマージし、異なる role URI の
    ツリーはそのまま結合する。

    Args:
        *tree_dicts: マージ対象の辞書群。

    Returns:
        マージ済み辞書。
    """



from edinet._config import configure as configure
from edinet._http import aclose as aclose
from edinet.api.cache import CacheInfo as CacheInfo, cache_info as cache_info, clear_cache as clear_cache
from edinet.exceptions import EdinetAPIError as EdinetAPIError, EdinetConfigError as EdinetConfigError, EdinetError as EdinetError, EdinetParseError as EdinetParseError, EdinetWarning as EdinetWarning
from edinet.financial.diff import DiffResult as DiffResult, diff_periods as diff_periods, diff_revisions as diff_revisions
from edinet.financial.dimensions.fiscal_year import FiscalYearInfo as FiscalYearInfo, detect_fiscal_year as detect_fiscal_year
from edinet.financial.dimensions.segments import extract_segments as extract_segments, list_dimension_axes as list_dimension_axes
from edinet.financial.extract import ExtractedValue as ExtractedValue, extract_values as extract_values, extracted_to_dict as extracted_to_dict
from edinet.financial.mapper import summary_mapper as summary_mapper, statement_mapper as statement_mapper, definition_mapper as definition_mapper, calc_mapper as calc_mapper, dict_mapper as dict_mapper
from edinet.financial.standards.canonical_keys import CK as CK
from edinet.financial.statements import Statements as Statements
from edinet.financial.summary import FilingSummary as FilingSummary, build_summary as build_summary
from edinet.models.company import Company as Company
from edinet.models.doc_types import DocType as DocType
from edinet.models.filing import Filing as Filing
from edinet.models.financial import FinancialStatement as FinancialStatement, LineItem as LineItem
from edinet.models.revision import RevisionChain as RevisionChain, build_revision_chain as build_revision_chain
from edinet.public_api import adocuments as adocuments, documents as documents
from edinet.xbrl.taxonomy.custom import CustomDetectionResult as CustomDetectionResult, detect_custom_items as detect_custom_items, find_custom_concepts as find_custom_concepts
from edinet.xbrl.validation.calc_check import CalcValidationResult as CalcValidationResult, validate_calculations as validate_calculations
from edinet.xbrl.text import extract_text_blocks as extract_text_blocks, build_section_map as build_section_map, clean_html as clean_html, TextBlock as TextBlock, SectionMap as SectionMap
from edinet.financial.notes.employees import extract_employee_info as extract_employee_info, EmployeeInfo as EmployeeInfo

__all__ = ['configure', 'documents', 'adocuments', 'aclose', 'Company', 'Filing', 'DocType', 'Statements', 'FinancialStatement', 'LineItem', 'clear_cache', 'cache_info', 'CacheInfo', 'build_revision_chain', 'RevisionChain', 'detect_custom_items', 'CustomDetectionResult', 'validate_calculations', 'CalcValidationResult', 'detect_fiscal_year', 'FiscalYearInfo', 'find_custom_concepts', 'diff_revisions', 'diff_periods', 'DiffResult', 'extract_segments', 'list_dimension_axes', 'build_summary', 'FilingSummary', 'EdinetError', 'EdinetConfigError', 'EdinetAPIError', 'EdinetParseError', 'EdinetWarning', 'CK', 'extract_values', 'extracted_to_dict', 'ExtractedValue', 'summary_mapper', 'statement_mapper', 'dict_mapper', 'extract_text_blocks', 'build_section_map', 'clean_html', 'TextBlock', 'SectionMap', 'extract_employee_info', 'EmployeeInfo']



"""XBRL バリデーションサブパッケージ。"""

from edinet.xbrl.validation.calc_check import (
    CalcValidationResult as CalcValidationResult,
    ValidationIssue as ValidationIssue,
    validate_calculations as validate_calculations,
)

__all__ = [
    "CalcValidationResult",
    "ValidationIssue",
    "validate_calculations",
]
